#!/usr/bin/env python3

# SPDX-FileCopyrightText: © 2022 Decompollaborate
# SPDX-License-Identifier: MIT

from __future__ import annotations

from .Elf32Constants import Elf32HeaderIdentifier, Elf32ObjectFileType, Elf32HeaderFlag, Elf32SectionHeaderType, Elf32SymbolTableType, Elf32SymbolTableBinding, Elf32SymbolVisibility, Elf32SectionHeaderNumber, Elf32DynamicTable
from .Elf32Dyns import Elf32Dyns, Elf32DynEntry
from .Elf32GlobalOffsetTable import Elf32GlobalOffsetTable
from .Elf32Header import Elf32Header
from .Elf32RegInfo import Elf32RegInfo
from .Elf32SectionHeaders import Elf32SectionHeaders, Elf32SectionHeaderEntry
from .Elf32StringTable import Elf32StringTable
from .Elf32Syms import Elf32Syms, Elf32SymEntry
from .Elf32Rels import Elf32Rels, Elf32RelEntry

from .Elf32File import Elf32File

# To avoid breaking backwards compatibility
from ..common.Relocation import RelocType as Elf32Relocs



#!/usr/bin/env python3

# SPDX-FileCopyrightText: © 2022 Decompollaborate
# SPDX-License-Identifier: MIT

from __future__ import annotations

from . import analysis

from .MipsSymbolBase import SymbolBase

from .MipsSymbolText import SymbolText
from .MipsSymbolData import SymbolData
from .MipsSymbolRodata import SymbolRodata
from .MipsSymbolBss import SymbolBss

from .MipsSymbolFunction import SymbolFunction



#!/usr/bin/env python3

# SPDX-FileCopyrightText: © 2022 Decompollaborate
# SPDX-License-Identifier: MIT

from __future__ import annotations

from .OrderedEnum import OrderedEnum


class FileSectionType(OrderedEnum):
    Unknown = -2
    Invalid = -1

    Text    = 1
    Data    = 2
    Rodata  = 3
    Bss     = 4
    Reloc   = 5

    @staticmethod
    def fromId(sectionId: int) -> FileSectionType:
        if sectionId == 1:
            return FileSectionType.Text
        if sectionId == 2:
            return FileSectionType.Data
        if sectionId == 3:
            return FileSectionType.Rodata
        if sectionId == 4:
            return FileSectionType.Bss
        if sectionId == 5:
            return FileSectionType.Reloc
        return FileSectionType.Invalid

    @staticmethod
    def fromStr(x: str) -> FileSectionType:
        if x == ".text":
            return FileSectionType.Text
        if x == ".data":
            return FileSectionType.Data
        if x == ".rodata":
            return FileSectionType.Rodata
        if x == ".rdata":
            return FileSectionType.Rodata
        if x == ".bss":
            return FileSectionType.Bss
        if x == ".reloc":
            return FileSectionType.Reloc
        return FileSectionType.Invalid

    def toStr(self) -> str:
        if self == FileSectionType.Text:
            return ".text"
        if self == FileSectionType.Data:
            return ".data"
        if self == FileSectionType.Rodata:
            return ".rodata"
        if self == FileSectionType.Bss:
            return ".bss"
        if self == FileSectionType.Reloc:
            return ".reloc"
        return ""

    def toCapitalizedStr(self) -> str:
        if self == FileSectionType.Text:
            return "Text"
        if self == FileSectionType.Data:
            return "Data"
        if self == FileSectionType.Rodata:
            return "RoData"
        if self == FileSectionType.Bss:
            return "Bss"
        if self == FileSectionType.Reloc:
            return "Reloc"
        return ""

    def toSectionName(self) -> str:
        if self == FileSectionType.Text:
            return ".text"
        if self == FileSectionType.Data:
            return ".data"
        if self == FileSectionType.Rodata:
            return ".rodata"
        if self == FileSectionType.Bss:
            return ".bss"
        if self == FileSectionType.Reloc:
            return ".ovl"
        return ""


FileSections_ListBasic = [FileSectionType.Text, FileSectionType.Data, FileSectionType.Rodata, FileSectionType.Bss]
FileSections_ListAll = [FileSectionType.Text, FileSectionType.Data, FileSectionType.Rodata, FileSectionType.Bss, FileSectionType.Reloc]



# SPDX-FileCopyrightText: © 2022 Decompollaborate
# SPDX-License-Identifier: MIT

from . import Utils

from .SortedDict import SortedDict
from .GlobalConfig import GlobalConfig, InputEndian, Compiler, Abi, ArchLevel, InputFileType
from .FileSectionType import FileSectionType, FileSections_ListBasic, FileSections_ListAll
from .ContextSymbols import SymbolSpecialType, ContextSymbol, gKnownTypes
from .SymbolsSegment import SymbolsSegment
from .Context import Context
from .FileSplitFormat import FileSplitFormat, FileSplitEntry
from .ElementBase import ElementBase
from .GlobalOffsetTable import GlobalOffsetTable
from .OrderedEnum import OrderedEnum
from .Relocation import RelocType, RelocationInfo, RelocationStaticReference



#!/usr/bin/env python3

# SPDX-FileCopyrightText: © 2022 Decompollaborate
# SPDX-License-Identifier: MIT

from __future__ import annotations

import dataclasses

from .. import common


@dataclasses.dataclass
class GotSymEntry:
    address: int
    contextSym: common.ContextSymbol|None = None

class GlobalOffsetTable:
    def __init__(self):
        self.localsTable: list[GotSymEntry] = list()
        self.globalsTable: list[GotSymEntry] = list()

        self.tableStart: int|None = None


    def initTables(self, pltGot: int, localsTable: list[int], globalsTable: list[int]):
        self.tableStart = pltGot
        self.localsTable = [GotSymEntry(address) for address in localsTable]
        self.globalsTable = [GotSymEntry(address) for address in globalsTable]


    def getGotSymEntry(self, address: int) -> tuple[GotSymEntry|None, bool|None]:
        if self.tableStart is None:
            return None, None

        index = (address - self.tableStart) // 4

        if index < 0:
            common.Utils.eprint(f"Warning: %got address 0x{address:X} not found on local or global table (negative index)")
            common.Utils.eprint(f"\tindex: {index}, len(localsTable):{len(self.localsTable)}, len(globalsTable): {len(self.globalsTable)}")
            return None, None

        if index < len(self.localsTable):
            return self.localsTable[index], False

        index -= len(self.localsTable)
        if index >= len(self.globalsTable):
            common.Utils.eprint(f"Warning: %got address 0x{address:X} not found on local or global table (out of range)")
            common.Utils.eprint(f"\tindex: {index}, len(globalsTable): {len(self.globalsTable)}")
            return None, None
        return self.globalsTable[index], True



#!/usr/bin/env python3

# SPDX-FileCopyrightText: © 2022 Decompollaborate
# SPDX-License-Identifier: MIT

from __future__ import annotations

from .MipsSectionBase import SectionBase

from .MipsSectionText import SectionText
from .MipsSectionData import SectionData
from .MipsSectionRodata import SectionRodata
from .MipsSectionBss import SectionBss
from .MipsSectionRelocZ64 import SectionRelocZ64, RelocEntry



#!/usr/bin/env python3

# SPDX-FileCopyrightText: © 2022 Decompollaborate
# SPDX-License-Identifier: MIT

from __future__ import annotations


from .SingleFileDisasmInternals import getToolDescription, addOptionsToParser, getArgsParser, applyArgs, applyGlobalConfigurations, getSplits, changeGlobalSegmentRanges, processArguments, addSubparser, disassemblerMain



#!/usr/bin/env python3

# SPDX-FileCopyrightText: © 2022 Decompollaborate
# SPDX-License-Identifier: MIT

from __future__ import annotations

from . import sections
from . import symbols

from .FuncRodataEntry import FunctionRodataEntry

from . import FilesHandlers

from .InstructionConfig import InstructionConfig
from .MipsFileBase import FileBase, createEmptyFile
from .MipsFileSplits import FileSplits
